from typing import Any, Dict, Optional, Union

import httpx
from pydantic import BaseModel, HttpUrl

from ..exceptions import HTTPException
from ..models import *


class AsyncClient(BaseModel):
    model_config = {"validate_assignment": True}

    base_url: str = "/"
    verify: Union[bool, str] = True
    access_token: Optional[str] = None

    def get_access_token(self) -> Optional[str]:
        return self.access_token

    def set_access_token(self, value: str) -> None:
        self.access_token = value

    async def get_schema_token_issuer_schema_get(self) -> Any:
        base_url = self.base_url
        path = f"/token-issuer/schema"

        headers = {
            "Content-Type": "application/json",
            "Accept": "application/json",
            "Authorization": f"Bearer { self.get_access_token() }",
        }

        query_params: Dict[str, Any] = {}
        query_params = {k: v for (k, v) in query_params.items() if v is not None}

        async with httpx.AsyncClient(base_url=base_url, verify=self.verify) as client:
            response = await client.request(
                "get",
                httpx.URL(path),
                headers=headers,
                params=query_params,
            )

        if response.status_code != 200:
            raise HTTPException(
                response.status_code,
                f"get_schema_token_issuer_schema_get failed with status code: {response.status_code}",
            )

        body = None if 200 == 204 else response.json()
        return body

    async def get_token_issuer__id__get(
        self,
        id: str,
    ) -> Any:
        base_url = self.base_url
        path = f"/token-issuer/{id}"

        headers = {
            "Content-Type": "application/json",
            "Accept": "application/json",
            "Authorization": f"Bearer { self.get_access_token() }",
        }

        query_params: Dict[str, Any] = {}
        query_params = {k: v for (k, v) in query_params.items() if v is not None}

        async with httpx.AsyncClient(base_url=base_url, verify=self.verify) as client:
            response = await client.request(
                "get",
                httpx.URL(path),
                headers=headers,
                params=query_params,
            )

        if response.status_code != 200:
            raise HTTPException(
                response.status_code,
                f"get_token_issuer__id__get failed with status code: {response.status_code}",
            )

        body = None if 200 == 204 else response.json()
        return body

    async def delete_one_token_issuer__id__delete(
        self,
        id: str,
    ) -> Any:
        base_url = self.base_url
        path = f"/token-issuer/{id}"

        headers = {
            "Content-Type": "application/json",
            "Accept": "application/json",
            "Authorization": f"Bearer { self.get_access_token() }",
        }

        query_params: Dict[str, Any] = {}
        query_params = {k: v for (k, v) in query_params.items() if v is not None}

        async with httpx.AsyncClient(base_url=base_url, verify=self.verify) as client:
            response = await client.request(
                "delete",
                httpx.URL(path),
                headers=headers,
                params=query_params,
            )

        if response.status_code != 204:
            raise HTTPException(
                response.status_code,
                f"delete_one_token_issuer__id__delete failed with status code: {response.status_code}",
            )

        body = None if 204 == 204 else response.json()
        return body

    async def update_token_issuer__id__patch(
        self,
        id: str,
        data: UpdateTokenIssuerRequest,
    ) -> Any:
        base_url = self.base_url
        path = f"/token-issuer/{id}"

        headers = {
            "Content-Type": "application/json",
            "Accept": "application/json",
            "Authorization": f"Bearer { self.get_access_token() }",
        }

        query_params: Dict[str, Any] = {}
        query_params = {k: v for (k, v) in query_params.items() if v is not None}

        async with httpx.AsyncClient(base_url=base_url, verify=self.verify) as client:
            response = await client.request(
                "patch",
                httpx.URL(path),
                headers=headers,
                params=query_params,
                json=data.dict(),
            )

        if response.status_code != 200:
            raise HTTPException(
                response.status_code,
                f"update_token_issuer__id__patch failed with status code: {response.status_code}",
            )

        body = None if 200 == 204 else response.json()
        return body

    async def create_token_issuer_post(
        self,
        data: CreateTokenIssuerRequest,
    ) -> Any:
        base_url = self.base_url
        path = f"/token-issuer"

        headers = {
            "Content-Type": "application/json",
            "Accept": "application/json",
            "Authorization": f"Bearer { self.get_access_token() }",
        }

        query_params: Dict[str, Any] = {}
        query_params = {k: v for (k, v) in query_params.items() if v is not None}

        async with httpx.AsyncClient(base_url=base_url, verify=self.verify) as client:
            response = await client.request(
                "post",
                httpx.URL(path),
                headers=headers,
                params=query_params,
                json=data.dict(),
            )

        if response.status_code != 200:
            raise HTTPException(
                response.status_code,
                f"create_token_issuer_post failed with status code: {response.status_code}",
            )

        body = None if 200 == 204 else response.json()
        return body



from typing import Annotated, Union

from pydantic import Field

from .PlaywrightCDPBrowserlessImpersonator_Input import PlaywrightCDPBrowserlessImpersonator_Input
from .PlaywrightCDPImpersonator import PlaywrightCDPImpersonator
from .PlaywrightImpersonator import PlaywrightImpersonator
from .TemplateImpersonator import TemplateImpersonator

Impersonator = Annotated[
    Union[
        PlaywrightImpersonator,
        PlaywrightCDPImpersonator,
        PlaywrightCDPBrowserlessImpersonator_Input,
        TemplateImpersonator,
    ],
    Field(discriminator="tool"),
]



from typing import Annotated, Union

from pydantic import Field

from .PKCEOAuth2Client import PKCEOAuth2Client
from .StandardOAuth2Client import StandardOAuth2Client

Oauth2Client = Annotated[Union[StandardOAuth2Client, PKCEOAuth2Client], Field(discriminator="type")]



from typing import *

from pydantic import BaseModel, Field


class OIDCConfig(BaseModel):
    """
    OIDCConfig model
    """

    model_config = {"populate_by_name": True, "validate_assignment": True}

    client_id: str = Field(validation_alias="client_id")

    client_secret: Optional[Union[str, None]] = Field(validation_alias="client_secret", default=None)

    redirect_uri: str = Field(validation_alias="redirect_uri")

    authorize_url: str = Field(validation_alias="authorize_url")

    token_url: str = Field(validation_alias="token_url")



from typing import Annotated, Union

from pydantic import Field

from .ImpersonationOAuth2Flow_Input import ImpersonationOAuth2Flow_Input
from .TemplateOAuth2Flow import TemplateOAuth2Flow

Oauth2Flow = Annotated[Union[ImpersonationOAuth2Flow_Input, TemplateOAuth2Flow], Field(discriminator="type")]



from typing import *

from pydantic import BaseModel, Field

from .TokenIssuer import TokenIssuer


class ManagedTokenIssuer(BaseModel):
    """
    ManagedTokenIssuer model
    """

    model_config = {"populate_by_name": True, "validate_assignment": True}

    updated_at: str = Field(validation_alias="updated_at")

    created_by: Optional[Union[str, None]] = Field(validation_alias="created_by", default=None)

    created_at: str = Field(validation_alias="created_at")

    id: Optional[str] = Field(validation_alias="id", default=None)

    name: str = Field(validation_alias="name")

    token_issuer: TokenIssuer = Field(validation_alias="token_issuer")



from typing import *

from pydantic import BaseModel, Field

from .TokenIssuer import TokenIssuer


class UpdateTokenIssuerRequest(BaseModel):
    """
    UpdateTokenIssuerRequest model
    """

    model_config = {"populate_by_name": True, "validate_assignment": True}

    id: str = Field(validation_alias="id")

    created_by: str = Field(validation_alias="created_by")

    name: str = Field(validation_alias="name")

    token_issuer: TokenIssuer = Field(validation_alias="token_issuer")



from typing import *

from pydantic import BaseModel, Field


class BrowserlessService_Input(BaseModel):
    """
    BrowserlessService model
        A minimal client for Browserless's session API.
    """

    model_config = {"populate_by_name": True, "validate_assignment": True}

    base_url: str = Field(validation_alias="base_url")



from enum import Enum


class ImpersonatorTool(str, Enum):
    PLAYWRIGHT = "PLAYWRIGHT"
    PLAYWRIGHT_CDP = "PLAYWRIGHT_CDP"
    PLAYWRIGHT_CDP_BROWSERLESS = "PLAYWRIGHT_CDP_BROWSERLESS"
    TEMPLATE = "TEMPLATE"



from typing import *
from typing import Literal

from pydantic import BaseModel, Field

from .ImpersonatorTool import ImpersonatorTool


class TemplateImpersonator(BaseModel):
    """
    TemplateImpersonator model
        Automate browser login to capture auth code via OIDC with PKCE.
    """

    model_config = {"populate_by_name": True, "validate_assignment": True}

    tool: Literal[ImpersonatorTool.TEMPLATE] = Field(validation_alias="tool", default=ImpersonatorTool.TEMPLATE)



from typing import *
from typing import Literal

from pydantic import BaseModel, Field

from .Oauth2FlowType import Oauth2FlowType


class TemplateOAuth2Flow(BaseModel):
    """
    TemplateOAuth2Flow model
        Automate browser login to capture auth code via OIDC with PKCE.
    """

    model_config = {"populate_by_name": True, "validate_assignment": True}

    idp_prefix: str = Field(validation_alias="idp_prefix")

    timeout: Optional[Union[int, None]] = Field(validation_alias="timeout", default=None)

    type: Literal[Oauth2FlowType.TEMPLATE] = Field(validation_alias="type", default=Oauth2FlowType.TEMPLATE)



from typing import *
from typing import Literal

from pydantic import BaseModel, Field

from .BrowserlessService_Input import BrowserlessService_Input
from .CDPGUIService import CDPGUIService
from .ImpersonatorTool import ImpersonatorTool


class PlaywrightCDPBrowserlessImpersonator_Input(BaseModel):
    """
    PlaywrightCDPBrowserlessImpersonator model
        Automate browser login to capture auth code via OIDC with PKCE. Good For CLIs, but
    """

    model_config = {"populate_by_name": True, "validate_assignment": True}

    tool: Literal[ImpersonatorTool.PLAYWRIGHT_CDP_BROWSERLESS] = Field(
        validation_alias="tool", default=ImpersonatorTool.PLAYWRIGHT_CDP_BROWSERLESS
    )

    cdp_endpoint: str = Field(validation_alias="cdp_endpoint")

    cdp_headers: Optional[Union[Dict[str, Any], None]] = Field(validation_alias="cdp_headers", default=None)

    cdp_timeout: Optional[Union[float, None]] = Field(validation_alias="cdp_timeout", default=None)

    cdp_gui_service: Optional[Union[CDPGUIService, None]] = Field(validation_alias="cdp_gui_service", default=None)

    browserless_service: BrowserlessService_Input = Field(validation_alias="browserless_service")



from .BrowserlessService_Input import *
from .BrowserlessService_Output import *
from .CDPGUIService import *
from .CreateTokenIssuerRequest import *
from .HTTPValidationError import *
from .ImpersonationOAuth2Flow_Input import *
from .ImpersonationOAuth2Flow_Output import *
from .ManagedTokenIssuer import *
from .OAuth2TokenIssuer_Input import *
from .OAuth2TokenIssuer_Output import *
from .OIDCConfig import *
from .PKCEOAuth2Client import *
from .PKCEOAuth2TokenIssuer_Input import *
from .PKCEOAuth2TokenIssuer_Output import *
from .PlaywrightCDPBrowserlessImpersonator_Input import *
from .PlaywrightCDPBrowserlessImpersonator_Output import *
from .PlaywrightCDPImpersonator import *
from .PlaywrightImpersonator import *
from .StandardOAuth2Client import *
from .TemplateImpersonator import *
from .TemplateOAuth2Flow import *
from .TemplateTokenIssuer_Input import *
from .TemplateTokenIssuer_Output import *
from .UpdateTokenIssuerRequest import *
from .ValidationError import *
from .TokenIssuer import *
from .Impersonator import *
from .Oauth2Flow import *
from .Oauth2Client import *
from .TokenIssuerType import *
from .ImpersonatorTool import *
from .Oauth2FlowType import *
from .Oauth2ClientType import *



from typing import *

from pydantic import BaseModel, Field


class BrowserlessService_Output(BaseModel):
    """
    BrowserlessService model
        A minimal client for Browserless's session API.
    """

    model_config = {"populate_by_name": True, "validate_assignment": True}

    base_url: str = Field(validation_alias="base_url")

    sessions_url: str = Field(validation_alias="sessions_url")

    ws_url_prefix: str = Field(validation_alias="ws_url_prefix")



from typing import *
from typing import Literal

from pydantic import BaseModel, Field

from .Oauth2ClientType import Oauth2ClientType
from .OIDCConfig import OIDCConfig


class PKCEOAuth2Client(BaseModel):
    """
    PKCEOAuth2Client model
    """

    model_config = {"populate_by_name": True, "validate_assignment": True}

    config: OIDCConfig = Field(validation_alias="config")

    type: Literal[Oauth2ClientType.PKCE] = Field(validation_alias="type", default=Oauth2ClientType.PKCE)



from typing import *
from typing import Literal

from pydantic import BaseModel, Field

from .Oauth2Client import Oauth2Client
from .Oauth2Flow import Oauth2Flow
from .TokenIssuerType import TokenIssuerType


class TemplateTokenIssuer_Output(BaseModel):
    """
    TemplateTokenIssuer model
    """

    model_config = {"populate_by_name": True, "validate_assignment": True}

    oauth2_flow: Oauth2Flow = Field(validation_alias="oauth2_flow")

    oauth2_client: Oauth2Client = Field(validation_alias="oauth2_client")

    identity_provider: Optional[str] = Field(validation_alias="identity_provider", default=None)

    response_type: Optional[str] = Field(validation_alias="response_type", default=None)

    scope: Optional[str] = Field(validation_alias="scope", default=None)

    type: Literal[TokenIssuerType.TEMPLATE] = Field(validation_alias="type", default=TokenIssuerType.TEMPLATE)



from typing import Annotated, Union

from pydantic import Field

from .OAuth2TokenIssuer_Input import OAuth2TokenIssuer_Input
from .PKCEOAuth2TokenIssuer_Input import PKCEOAuth2TokenIssuer_Input
from .TemplateTokenIssuer_Input import TemplateTokenIssuer_Input

TokenIssuer = Annotated[
    Union[PKCEOAuth2TokenIssuer_Input, OAuth2TokenIssuer_Input, TemplateTokenIssuer_Input], Field(discriminator="type")
]
